
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "codegen.h"
#include "stack.h"
#include "symtab.h"


extern FILE *output;
extern stack register_stack;

int free_reg_num = 0;


void indent(bool tabs) {
    code("\n");
    if(tabs)
    code("\t\t\t");
}


// REGISTERS

int take_reg(void) {
    if(free_reg_num >= LAST_WORKING_REGISTER + 1) {
        syyerror("Compiler error! No free registers!");
        exit(EXIT_FAILURE);
    }
    return free_reg_num++;
}

void free_reg(void) {
    if(free_reg_num-- < 0) {
        syyerror("Compiler error! No more registers to free!");
        exit(EXIT_FAILURE);
    }
    else
        set_register_type(free_reg_num, NO_TYPE);
}

// Ako je u pitanju indeks registra, oslobodi registar
void release_reg(int reg_index) {
    if(reg_index >= 0 && reg_index <= LAST_WORKING_REGISTER)
        free_reg();
}


// LABELS & JUMPS

void gen_str_lab(char *str1, char *str2, bool prefix) {
    indent(FALSE);
    if(prefix)
        code("@");
    code("%s%s:", str1, str2);
}

void gen_num_lab(char *str, int num, bool prefix) {
    indent(FALSE);
    if(prefix)
        code("@");
    code("%s%d:", str, num);
}

void gen_jump_to_str_lab(char *jmp, char *str1, char *str2, bool prefix) {
    indent(TRUE);
    code("%s\t", jmp);
    if(prefix)
        code("@");
    code("%s%s", str1, str2);
}

void gen_jump_to_num_lab(char *jmp, char *str, int num, bool prefix) {
    indent(TRUE);
    code("%s\t", jmp);
    if(prefix)
        code("@");
    code("%s%d", str, num);
}

// SYMBOL
void print_symbol(int index) {
    if(index > -1) {
        // - n * 4 (%14)
        if(get_kind(index) == LOCAL_VAR)
            code("-%d(%%14)", get_attribute(index) * 4);

        // m * 4 (%14)
        else 
            if(get_kind(index) == PARAMETER)
                code("%d(%%14)", 4 + get_attribute(index) *4);

            // identifier
            else
                if(get_kind(index) == CONSTANT)
                    code("$%s", get_name(index));
                else
                    code("%s", get_name(index));
    }
}


// OTHER

void gen_cmp(int operand1_index, int operand2_index) {
    release_reg(operand1_index);
    release_reg(operand2_index);
    indent(TRUE);
    code("%s \t", cmps[get_type(operand1_index) - 1]);
    print_symbol(operand1_index);
    code(",");
    print_symbol(operand2_index);
}

void gen_mov(int input_index, int output_index) {
    release_reg(input_index);
    indent(TRUE);
    code("MOV \t");
    print_symbol(input_index);
    code(",");
    print_symbol(output_index);

    //ako smestas u registar, prenesi tip 
    if(output_index >= 0 && output_index <= LAST_WORKING_REGISTER)
      set_register_type(output_index, get_type(input_index));
}


// STATEMENTS
int gen_arith(int statement, int operand1_index, int operand2_index) {
    int output_index;
    release_reg(operand1_index);
    release_reg(operand2_index);
    output_index = take_reg();

    indent(TRUE);
    code("%s\t", arithmetic_operators[get_type(operand1_index) - 1][statement]);
    print_symbol(operand1_index);
    code(",");
    print_symbol(operand2_index);
    code(",");
    print_symbol(output_index);

    //tip izraza = tip jednog od operanada (recimo prvog)
    set_register_type(output_index, get_type(operand1_index));
    return output_index;
}


// FUNCTION

void gen_function_call(int name_index) {
    indent(TRUE);
    code("CALL\t");
    print_symbol(name_index);
}

void gen_clear_loc_var(int num) {
    if(num > 0) {
    indent(TRUE);
        code("ADDU\t%%15,$%d,%%15", num * 4);
    }
}

void gen_push(int arg_index) {
    release_reg(arg_index);
    indent(TRUE);
    code("PUSH\t");
    print_symbol(arg_index);
}

void gen_reg_save() {
    int i;
    for(i = 0; i < free_reg_num; i++) {
    indent(TRUE);
        code("PUSH\t%%%d", i);
    }
    push(&register_stack, free_reg_num);
    free_reg_num = 0;
}

void gen_reg_restore() {
    int i;
    free_reg_num = pop(&register_stack);
    for(i = free_reg_num - 1; i >= 0; i--) {
    indent(TRUE);
        code("POP\t%%%d", i);
    }
}


