/* Leksicki analizator za microC */
/*(vraca tokene, a main funkcija ih zatim stampa) */

%{
    #include <string.h>
    #include "defs.h"
    #define YYSTYPE int

    void print_token(char *string, int token, int value);

    YYSTYPE yylval;
    int line = 1;

    static char* tokens[] = { "", "_TYPE", "_IF", "_ELSE", "_RETURN", "_ID", "_INT_NUMBER", "_UNSIGNED_NUMBER", 
                              "_LPAREN", "_RPAREN", "_COMMA", "_LBRACKET", "_RBRACKET", "ASSIGN", "_SEMICOLON",
                              "_PLUS", "_MINUS", "_TIMES", "_DIV", "_RELOP" };
    static char* types[]  = { "INT_TYPE", "UNSIGNED_TYPE" };
    static char* relops[] = { "LT", "GT", "LE", "GE", "EQ", "NE" };
%}

letter                      [_a-zA-Z]
digit                       [0-9]

%%

[ \t]+                      {   /* skip */    }

"int"                       {   yylval = INT_TYPE; return _TYPE;        }
"unsigned"                  {   yylval = UNSIGNED_TYPE; return _TYPE;   }
"if"                        {   return _IF;                             }
"else"                      {   return _ELSE;                           }
"return"                    {   return _RETURN;                         }

{letter}({letter}|{digit})* {
                                yylval = (int)strdup(yytext);
                                return _ID; 
                            }


{digit}{1,5}[uU]            {
                                char *s = strdup(yytext);
                                  //skini slovo "u" sa kraja
                                s[strlen(s)-1] = 0;
                                yylval = (int)s; 
                                return _UNSIGNED_NUMBER; 
                            }

[+-]?{digit}{1,5}           {
                                yylval = (int)strdup(yytext);
                                return _INT_NUMBER;
                            }


"("                         {   return _LPAREN;                  }
")"                         {   return _RPAREN;                  }
","                         {   return _COMMA;                   }
"{"                         {   return _LBRACKET;                }
"}"                         {   return _RBRACKET;                }
"="                         {   return _ASSIGN;                  }
";"                         {   return _SEMICOLON;               }
"+"                         {   return _PLUS;                    }
"-"                         {   return _MINUS;                   }
"*"                         {   return _TIMES;                   }
"/"                         {   return _DIV;                     }
"<"                         {   yylval = LT; return _RELOP;      }
">"                         {   yylval = GT; return _RELOP;      }
"<="                        {   yylval = LE; return _RELOP;      }
">="                        {   yylval = GE; return _RELOP;      }
"=="                        {   yylval = EQ; return _RELOP;      }
"!="                        {   yylval = NE; return _RELOP;      }


"/*"                        {
                                int in_comment = 1;
                                char c;
                                while(in_comment) {
                                    c = input(); 
                                    if(c == '*') {
                                        char ch = input();
                                        if(ch == '/') in_comment = 0;
                                        else unput(ch);
                                    }
                                    else if(c == '\n') line++;
                                    else if(c == EOF) {
                                        printf("\nLEXICAL ERROR: Unclosed comment in line %d.\n", line);
                                        in_comment = 0;
                                    }
                                }
                            }

"\n"                        {   line++;    }
.                           {   printf("\nLEXICAL ERROR on character %c (line %d)", yytext[0], line);   }

%%

int yywrap() {
    return 1;
}

int main() {
    int token;
    printf("\nSTART\n");
    while( (token = yylex()) != 0) {
        print_token(yytext, token, yylval);
    }
    printf("\nSTOP\n");
    return 0;
}

void print_token(char *string, int token, int value) {
    printf("\n%16s  TOKEN: %16s  ", string, tokens[token]);
    switch(token) {

        //yylval = string
        case _ID :
        case _INT_NUMBER :
        case _UNSIGNED_NUMBER : {
            printf("value: %s", (char *)value);
            free((char *)value);
        }; break;

        //yylval = number
        case _TYPE : {
            printf("value: %s", types[value]);
        }; break;
        case _RELOP : {
            printf("value: %s", relops[value]);
        }; break;
    }
}
