
#include <stdio.h>
#include "defs.h"
 

void parser(void);
void requesting(TOKENTYPE requesting_set, TOKENTYPE sync_set);
void print(TOKENTYPE token);
bool found(TOKENTYPE possible_set);
 
void program(void);
void variable_list(void);
void variable(void);
void procedure_list(void);
void procedure(void);
void statement_list(void);
void statement(void);
void assignment_or_procedure_call_statement(void);
void expression(void);
void operand(void);
void if_statement(void);
void condition(void);
void while_statement(void);

TOKENTYPE scanner(void);


TOKENTYPE token;
extern int line;



void parser(void) {
    printf("\nSTART\n\n");
    token = scanner();
    program();
    printf("\nSTOP\n");
}


void requesting(TOKENTYPE requesting_set, TOKENTYPE sync_set) {
    if(token IN requesting_set) {
        print(token);
        token = scanner();
    }
    else {
    printf("PARSING ERROR - EXPECTED ");
    print(requesting_set);
    while(!(token IN sync_set)) {
        if(token IN requesting_set) {
        print(token);
        token = scanner();
        break;
        }
        else {
        printf("SKIPED ");
        print(token);
        token = scanner();
        }
    }
    }
}

void print(TOKENTYPE token) {
    TOKENTYPE mask = 1;
    int i = 0;
    int prints = 0;

    for(i = 0; i < sizeof(TOKENTYPE)*8; i++) {
    if(token & mask) {
        if(prints > 0)
        printf(" or ");
        printf("%s",tokens[i+1]);
        prints++;
    }
    mask <<= 1;
    }
    if(prints == 0)
    printf("%s",tokens[NONE]);

    printf(" in line %d\n", line);
}

bool found(TOKENTYPE possible_set) {
    if(token IN possible_set) {
        print(token);
        token = scanner();
        return TRUE;
    }
    else
        return FALSE;
}



void program(void) {
    requesting(_PROGRAM, _VAR | _PROC | _BEGIN | _SEMICOLON | _EOF);
    requesting(_IDENTIFIER, _VAR | _PROC | _BEGIN | _SEMICOLON | _EOF);
    requesting(_SEMICOLON, _VAR | _PROC | _BEGIN | _EOF);
    if(found(_VAR))
        variable_list();
    if(found(_PROC))
        procedure_list();
    requesting(_BEGIN, _END | _SEMICOLON | _EOF);
    statement_list();
    requesting(_END, _EOF);
    printf("PROGRAM\n");
}

void variable_list(void) {
    variable();
    while(found(_SEMICOLON))
        variable();
    printf("VARIABLE_LIST\n");
}

void variable(void) {
    requesting(_IDENTIFIER, _COLON | _INT | _SEMICOLON | _PROC | _BEGIN
           | _EOF);
    requesting(_COLON, _INT | _SEMICOLON | _PROC | _BEGIN  | _EOF);
    requesting(_INT, _SEMICOLON | _PROC | _BEGIN  | _EOF);
    printf("VARIABLE\n");
}

void procedure_list(void) {
    procedure();
    while(found(_SEMICOLON)) {
        requesting(_PROC, _VAR | _BEGIN | _END);
        procedure();
    }
    printf("PROCEDURE_LIST\n");
}

void procedure(void) {
    requesting(_IDENTIFIER, _LEFT_PAR | _RIGHT_PAR | _VAR | _BEGIN | _END
           | _SEMICOLON | _EOF);
    if(found(_LEFT_PAR)) {
        variable_list();
        requesting(_RIGHT_PAR, _VAR | _BEGIN | _END | _SEMICOLON | _EOF);
    }
    requesting(_SEMICOLON, _VAR | _BEGIN | _END | _EOF);
    if(found(_VAR))
        variable_list();
    requesting(_BEGIN, _END | _SEMICOLON | _EOF);
    statement_list();
    requesting(_END, _BEGIN | _PROC | _SEMICOLON | _EOF);
    printf("PROCEDURE\n");
}

void statement_list(void) {
    statement();
    while(found(_SEMICOLON))
        statement();
    printf("STATEMENT_LIST\n");
}

void statement(void) {
    if(found(_IDENTIFIER))    
        assignment_or_procedure_call_statement();
    else {
        if(found(_IF)) 
            if_statement();
        else {
            if(found(_WHILE))
                while_statement();
            else
                if(found(_BEGIN)) {
                    statement_list();
                    requesting(_END, _SEMICOLON | _EOF);
                }
                else
                    if(token == _END)
                        printf("EMPTY STATEMENT\n");
                    else {
                        printf("PARSING ERROR - EXPECTED STATEMENT\n");
                    }
        }
    }
    printf("STATEMENT\n");
}

void assignment_or_procedure_call_statement(void) {
    if(found(_ASSIGN)) {
        expression();
        printf("ASSIGNMENT_STATEMENT\n");
    }
    else {
        if(found(_LEFT_PAR)) {
            expression();
            while(found(_COMMA))
                expression();
            requesting(_RIGHT_PAR, _SEMICOLON | _BEGIN | _END | _EOF);
            printf("PROCEDURE_CALL_STATEMENT\n");
        }
        else {
            printf("PROCEDURE_CALL_STATEMENT\n");
        }
    }
}

void expression(void) {
    found(_PLUS | _MINUS);
    operand();
    while(found(_MULTIPLY | _DIVIDE))
        operand();
    while(found(_PLUS | _MINUS)) {
        operand();
        while(found(_MULTIPLY | _DIVIDE))
            operand();
    }
    printf("EXPRESSION\n");
}

void operand(void) {
    if(!found(_CONSTANT | _IDENTIFIER)) {
        requesting(_LEFT_PAR, _RIGHT_PAR | _SEMICOLON |  _BEGIN | _END | _EOF);
        expression();
        requesting(_RIGHT_PAR, _SEMICOLON | _BEGIN | _END | _EOF);
    }
    printf("OPERAND\n");
}

void if_statement(void) {
    condition();
    requesting(_THEN, _ELSE | _SEMICOLON | _BEGIN | _END | _EOF);
    statement();
    if(found(_ELSE))
        statement();
    printf("IF_STATEMENT\n");
}

void condition(void) {
    expression();
    requesting(_LT | _GT | _LE | _GE | _EQ | _NE, _SEMICOLON | _BEGIN | _END
           | _EOF);
    expression();
    printf("CONDITION\n");
}

void while_statement(void) {
    condition();
    requesting(_DO, _SEMICOLON | _BEGIN | _END | _EOF);
    statement();
    printf("WHILE_STATEMENT\n");
}




int main(void) {
    parser();
    return 0;
}





