/* Parser koji broji reci i recenice */

%{
#include <stdio.h>
#include <stdlib.h>

#define YYSTYPE char*

int yylex(void);
int yyparse(void);

void print_indent(void);
void print(char *str);
void print_quoted(char *str);
void print_epsilon();

extern int line;
int word_counter = 0;
int sentence_counter = 0;
int indent = 0;
%}

%token    _DOT
%token    _CAPITAL_WORD
%token    _WORD


%%

text 
    :    /* empty text */
            {   print_epsilon();
	        print("text"); }
    |    text sentence
            {   print("text"); }
    ;
          
sentence 
    :    _CAPITAL_WORD 
            {   word_counter++;
	        print_quoted($1);
		print("_CAPITAL_WORD");
		free($1); }
         words _DOT
            {   sentence_counter++;
                print_quoted(".");
                print("_DOT");
                print("sentence"); }
    ;

words 
    :    /* empty */
            {   print_epsilon();
                print("words"); }

    |    words _WORD
            {   word_counter++; 
                print_quoted($2);
                print("_WORD");
                print("words");
                free($2); }
            
    |    words _CAPITAL_WORD    
            {   word_counter++;
	        print_quoted($2);
                print("_WORD");
                print("words");
                free($2); }
    ;


%%

int main() {
    printf("\nSTART\n");
    yyparse();
    printf("\nTotal words: %d.", word_counter);
    printf("\nTotal sentences: %d.\n", sentence_counter);
    printf("\nSTOP\n");
    return 0;
}


int yyerror(char *s) {
    fprintf(stderr, "\nSYNTAX ERROR in line %d: %s", line, s);
    return 0;
} 


void print_indent() {
    int i;
    printf("\n");
    for(i = 0; i < indent; i++)
	printf("    ");
}

void print(char *str) {
    printf("\n");
    print_indent();
    printf("%s",str);
}

void print_quoted(char *str) {
    indent++;
    print_indent();
    printf("\"%s\"", str);
}

void print_epsilon() {
    indent++;
    print_indent();
    printf("<epsilon>");
}
