/* Skener za obican tekst */

%{
#include <ctype.h>
#include <string.h>
#define YYSTYPE char*

void print_token(char *string, int token, char *value);

int line = 1;
YYSTYPE yylval;

#define _DOT             1
#define _CAPITAL_WORD    2
#define _WORD            3

static char* tokens[] = { "", 
                          "_DOT         ", 
                          "_CAPITAL_WORD", 
                          "_WORD        " };
%}


ws                  [ \t]+

capital_letter      [A-Z]
small_letter        [a-z]

word                {small_letter}+
capital_word        {capital_letter}{small_letter}*


%%

{ws}                { /* skip */ }

"."                 { return _DOT; }
{capital_word}      { yylval = strdup(yytext); return _CAPITAL_WORD; }
{word}              { yylval = strdup(yytext); return _WORD; }
                     
"\n"                { line++; }    
.                   { printf("\nLEXICAL ERROR on character %c in line %d", 
                              yytext[0], line); }    


%%

int main() {
    int token = -1;
    printf("\nSTART\n");
    
    while((token = yylex()) != 0) {
        print_token(yytext, token, yylval);
    }
    
    printf("\n\nSTOP\n");
    return 0;
}

void print_token(char *string, int token, char *value) {
    printf("\n%16s  TOKEN:  %s", string, tokens[token]);
    switch(token) { //ako je popunjena yylval, ispisi je
        case _CAPITAL_WORD :
        case _WORD : {
            printf("  value:  %s", value);
            free(value);
        }; break;
    }
}

int yywrap() {
    return 1;
}
